# Copyright (C) 2007, Red Hat, Inc.
# Copyright (C) 2012, Daniel Francis
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

"""
The toggle toolbutton is a widget similar to the normal toolbutton; it displays
and icon and fits in a toolbar.  However, instead of being clicked, it is
toggled between an on and off state.  In the on state, the button is
highlighted.
"""

import logging

from gi.repository import GObject
from gi.repository import Gtk

from sugar3.graphics.icon import Icon
from sugar3.graphics.palette import Palette, ToolInvoker


def _add_accelerator(tool_button):
    if not tool_button.props.accelerator or not tool_button.get_toplevel() or \
            not tool_button.get_child():
        return

    # TODO: should we remove the accelerator from the prev top level?
    if not hasattr(tool_button.get_toplevel(), 'sugar_accel_group'):
        logging.debug('No Gtk.AccelGroup in the top level window.')
        return

    accel_group = tool_button.get_toplevel().sugar_accel_group
    keyval, mask = Gtk.accelerator_parse(tool_button.props.accelerator)
    # the accelerator needs to be set at the child, so the Gtk.AccelLabel
    # in the palette can pick it up.
    accel_flags = Gtk.AccelFlags.LOCKED | Gtk.AccelFlags.VISIBLE
    tool_button.get_child().add_accelerator('clicked', accel_group,
                                            keyval, mask, accel_flags)


def _hierarchy_changed_cb(tool_button, previous_toplevel):
    _add_accelerator(tool_button)


def setup_accelerator(tool_button):
    _add_accelerator(tool_button)
    tool_button.connect('hierarchy-changed', _hierarchy_changed_cb)


class ToggleToolButton(Gtk.ToggleToolButton):
    '''
    A toggle toolbutton is a toolbar button that is toggled between the 
    on and off states.  In the "on" state, the background is highlighted.

    Keyword Args:
        icon_name (str): name to pass to :any:`ToggleToolButton.set_icon`
    '''

    __gtype_name__ = 'SugarToggleToolButton'

    def __init__(self, icon_name=None):
        GObject.GObject.__init__(self)

        self._palette_invoker = ToolInvoker(self)

        if icon_name:
            self.set_icon_name(icon_name)

        self.connect('destroy', self.__destroy_cb)

    def __destroy_cb(self, icon):
        if self._palette_invoker is not None:
            self._palette_invoker.detach()

    def set_icon_name(self, icon_name):
        '''
        Set the icon that is is visible in the button.  The icon name should
        be as you would pass to :any:`sugar3.graphics.icon.Icon.set_icon_name`.
        For example, "computer-xo" or "document-generic" are valid names.

        Args:
            icon_name (str): icon name to set
        '''
        icon = Icon(icon_name=icon_name)
        self.set_icon_widget(icon)
        icon.show()

    def get_icon_name(self):
        '''
        Returns:
            str, icon name passed to :any:`set_icon_name`
        '''
        if self.props.icon_widget is not None:
            return self.props.icon_widget.props.icon_name
        else:
            return None

    icon_name = GObject.property(type=str, setter=set_icon_name,
                                 getter=get_icon_name)

    def create_palette(self):
        '''
        Override this method to return a palette.  This method will be called
        just before a palette is needed, eg. when the user right clicks on
        the button.

        Also see:  :any:`set_palette` for pre-setting the palette.

        Returns:
            sugar3.graphics.palette.Palette or None
        '''
        return None

    def get_palette(self):
        '''
        Get the current palette, either set via :any:`set_palette` or
        :any:`create_palette` (it is has been called yet)

        Returns:
            sugar3.graphics.palette.Palette or None
        '''
        return self._palette_invoker.palette

    def set_palette(self, palette):
        '''
        Set the palette that will be visible when the user right clicks on
        the button.

        Also see:  :any:`create_palette` if you want to lazily create the
        palette.

        Args:
            palette (sugar3.graphics.palette.Palette or None)
        '''
        self._palette_invoker.palette = palette

    palette = GObject.property(
        type=object, setter=set_palette, getter=get_palette)

    def get_palette_invoker(self):
        return self._palette_invoker

    def set_palette_invoker(self, palette_invoker):
        self._palette_invoker.detach()
        self._palette_invoker = palette_invoker

    palette_invoker = GObject.property(
        type=object, setter=set_palette_invoker, getter=get_palette_invoker)

    def set_tooltip(self, text):
        '''
        Sets a tooltip palette for the button.  This will over-ride any
        palette set via :any:`set_palette` or :any:`create_palette`.

        Args:
            tooltip (str): short text describing button
        '''
        self.set_palette(Palette(text))

    def set_accelerator(self, accelerator):
        self._accelerator = accelerator
        setup_accelerator(self)

    def get_accelerator(self):
        return self._accelerator

    accelerator = GObject.property(type=str, setter=set_accelerator,
                                   getter=get_accelerator)

    def do_draw(self, cr):
        if self.palette and self.palette.is_up():
            allocation = self.get_allocation()
            # draw a black background, has been done by the engine before
            cr.set_source_rgb(0, 0, 0)
            cr.rectangle(0, 0, allocation.width, allocation.height)
            cr.paint()

        Gtk.ToggleToolButton.do_draw(self, cr)

        if self.palette and self.palette.is_up():
            invoker = self.palette.props.invoker
            invoker.draw_rectangle(cr, self.palette)

        return False

    def do_clicked(self):
        if self.palette:
            self.palette.popdown(True)

    palette = property(get_palette, set_palette)
